## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    comment = "#>"
)

## ----`install bioc`, eval=FALSE-----------------------------------------------
# if (!requireNamespace("BiocManager")) {
#     install.packages("BiocManager")
# }
# BiocManager::install("BatchSVG")

## ----'install dev', eval = FALSE----------------------------------------------
# remotes::install("christinehou11/BatchSVG")

## ----'library', message=FALSE-------------------------------------------------
library(BatchSVG)
# library(humanHippocampus2024)
library(ExperimentHub)
library(SpatialExperiment)
library(SummarizedExperiment)
library(tidyr)
library(dplyr)
library(tibble)
library(cowplot)

## ----load data, comment=NA, warning=FALSE, message=FALSE----------------------
ehub <- ExperimentHub()

# Load the datasets of the package
# myfiles <- query(ehub, "humanHippocampus2024")
# Resulting humanHippocampus2024 datasets from ExperimentHub query
# myfiles
# ExperimentHub with 2 records
# # snapshotDate(): 2024-10-24
# # $dataprovider: Lieber Institute for Brain Development (LIBD)
# # $species: Homo sapiens
# # $rdataclass: SpatialExperiment, SingleCellExperiment
# # additional mcols(): taxonomyid, genome, description,
# #   coordinate_1_based, maintainer, rdatadateadded, preparerclass, tags,
# #   rdatapath, sourceurl, sourcetype 
# # retrieve records with, e.g., 'object[["EH9605"]]' 
# 
#            title
#   EH9605 | spe  
#   EH9606 | sce
#   
# spe <- myfiles[["EH9605"]]

spe <- ehub[["EH9605"]]
spe

## ----subset, message=FALSE, warning=FALSE, comment=NA-------------------------
fix_order <- distinct(
    as.data.frame(colData(spe)), slide, array, brnum, sample_id, 
    position, sex) %>% 
    arrange(slide, array)
sub4 <- fix_order$sample_id[c(14,16, 20,21)]

spe_sub4 <- spe[,spe$sample_id %in% sub4]
spe_sub4 # 31483, 18945

## ----svgs_sub4, message=FALSE, warning=FALSE, comment=NA, eval=FALSE----------
# # res_ranks: SVGs results with rank values
# res_df_sub <- pivot_longer(
#     rownames_to_column(as.data.frame(res_ranks), var<-"gene_id"),
#         colnames(res_ranks),
#     names_to="sample_id",
#     values_to="rank",
#     values_drop_na=TRUE)
# 
# res_df_sub <- filter(res_df_sub,
#     sample_id %in%
#         c("V11L05-333_B1", "V11L05-333_D1", "V11L05-335_D1", "V11L05-336_A1"),
#     rank <= 2000) # top 2k sig features
# 
# svgs_sub4 <- group_by(res_df_sub, gene_id) |>
#     tally() |>
#     filter(n>1)

## ----nrow svgs_sub4, message=FALSE, warning=FALSE, comment=NA, eval=FALSE-----
# nrow(svgs_sub4)

## ----load svgs_sub4, message=FALSE, warning=FALSE, comment=NA, echo=FALSE-----
svgs_sub4 <- read.csv(
    system.file("extdata","svgs_sub4.csv",package = "BatchSVG"),
    row.names = 1, check.names = FALSE)

nrow(svgs_sub4)

## ----feature select, comment = NA, warning=FALSE------------------------------
spe_sub4 <- spe_sub4[rowData(spe_sub4)$gene_id %in% svgs_sub4$gene_id,]
rownames(spe_sub4) <- rowData(spe_sub4)$gene_id

SVGs <- svgs_sub4$gene_id
list_batch_df <- featureSelect(input = spe_sub4, 
    batch_effect = c("sample_id", "sex"), VGs = SVGs)

## ----feature select class, comment = NA, warning=FALSE------------------------
class(list_batch_df)

## ----feature select head sample id, comment = NA, warning=FALSE---------------
head(list_batch_df$sample_id)

## ----feature select head sex, comment = NA, warning=FALSE---------------------
head(list_batch_df$sex)

## ----svg, comment=NA, warning=FALSE, message=FALSE----------------------------
plots <- svg_nSD(list_batch_df = list_batch_df, 
                sd_interval_dev = c(5,4), sd_interval_rank = c(4,6))

## ----svg sample, warning=FALSE, message=FALSE, fig.width=10, fig.height=10----
plots$sample_id

## ----svg sex, warning=FALSE, message=FALSE, fig.width=10, fig.height=10-------
plots$sex

## ----svg one plot, warning=FALSE, message=FALSE,fig.width=10, fig.height=10----
plots <- svg_nSD(list_batch_df = list_batch_df[1], 
                sd_interval_dev = 5, sd_interval_rank = 7)
plots$sample_id

## ----bias detect dev, comment = NA, message=FALSE, warning=FALSE--------------
bias_dev <- biasDetect(list_batch_df = list_batch_df, 
    threshold = "dev", nSD_dev = 7)

## ----bias detect dev table, comment = NA, message=FALSE, warning=FALSE--------
head(bias_dev$sample_id$Table)

## ----bias detect dev plot, message=FALSE, warning=FALSE, fig.height=3---------
bias_dev$sample_id$Plot

## ----size change, message=FALSE, warning=FALSE, fig.width= 10, fig.height=6----
# size default = 3
bias_dev_size <- biasDetect(list_batch_df = list_batch_df, 
    threshold = "dev", nSD_dev = 7, plot_point_size = c(2,4))

plot_grid(bias_dev_size$sample_id$Plot,bias_dev_size$sex$Plot)

## ----bias detect rank, comment = NA, message=FALSE, warning=FALSE-------------
bias_rank <- biasDetect(list_batch_df = list_batch_df, 
    threshold = "rank", nSD_rank = 6)

## ----bias detect rank table, comment = NA, message=FALSE, warning=FALSE-------
head(bias_rank$sex$Table)

## ----bias detect rank plot, fig.height=3, message=FALSE, warning=FALSE--------
bias_rank$sex$Plot

## ----shape change, message=FALSE, warning=FALSE, fig.width= 10, fig.height=6----
# shape default = 16
bias_rank_shape <- biasDetect(list_batch_df = list_batch_df, 
    threshold = "rank", nSD_rank = 6, plot_point_shape = c(2, 18))

plot_grid(bias_rank_shape$sample_id$Plot,bias_rank_shape$sex$Plot)

## ----both, comment = NA, message=FALSE, warning=FALSE-------------------------
bias_both <- biasDetect(list_batch_df = list_batch_df, threshold = "both",
    nSD_dev = 7, nSD_rank = 6)

## ----both plot,message=FALSE,warning=FALSE,fig.width=10, fig.height=6---------
bias_both$sample_id$Plot

## ----both table, comment = NA, message=FALSE, warning=FALSE-------------------
head(bias_both$sex$Table)

## ----color change, message=FALSE, warning=FALSE, fig.width= 10, fig.height=8----
# color default = "YlOrRd"
bias_both_color <- biasDetect(list_batch_df = list_batch_df, 
    threshold = "both", nSD_dev = 7, nSD_rank = 6, plot_pallete = "Greens")

plot_grid(bias_both_color$sample_id$Plot,bias_both_color$sex$Plot,nrow = 2)

## ----text change, message=FALSE, warning=FALSE, fig.width= 10, fig.height=8----
# text size default = 3
bias_both_color_text <- biasDetect(list_batch_df = list_batch_df, 
    threshold = "both", nSD_dev = 7, nSD_rank = 6, 
    plot_pallete = c("Blues","Greens"), plot_text_size = c(2,4))

plot_grid(bias_both_color_text$sample_id$Plot,
    bias_both_color_text$sex$Plot,nrow = 2)

## ----new svgs, comment = NA, message=FALSE, warning=FALSE---------------------
bias_both_df <- bias_both$sample_id$Table
svgs_filt <- setdiff(svgs_sub4$gene_id, bias_both_df$gene_id)
svgs_sub4_filt <- svgs_sub4[svgs_sub4$gene_id %in% svgs_filt, ]
nrow(svgs_sub4_filt)

## ----'sessionInfo'------------------------------------------------------------
## Session info
sessionInfo()

